<?php
/**
 * Custom post type: Video
 *
 * @package EV Häs, LLC
 * @author E-intersol Marketing Solutions
 * @link https://www.eintersol.com
 */

if (! defined('ABSPATH')) {
	exit; // Exit if accessed directly
}

if (! class_exists('Mfn_Post_Type_Video')) {
	class Mfn_Post_Type_Video extends Mfn_Post_Type
	{

		/**
		 * Mfn_Post_Type_Video constructor
		 */

		public function __construct()
		{
			parent::__construct();

			// fires after WordPress has finished loading but before any headers are sent
			add_action('init', array($this, 'register'));

			// applied to the list of columns to print on the manage posts screen for a custom post type
			add_filter('manage_edit-video_columns', array($this, 'add_columns'));

			// allows to add or remove (unset) custom columns to the list post/page/custom post type pages
			add_action('manage_posts_custom_column', array($this, 'custom_column'));

			// admin only methods

			if( is_admin() ){
				$this->fields = $this->set_fields();
				$this->builder = new Mfn_Builder_Admin();
			}

		}

		/**
		 * Set post type fields
		 */

		private function set_fields(){

			return array(

				'id' => 'mfn-meta-video',
				'title' => esc_html__('Video Item Options', 'mfn-opts'),
				'page' => 'video',
				'fields' => array(					

					// media

					array(
						'id' => 'mfn-meta-info-media',
						'type' => 'info',
						'title' => '',
						'desc' => __('Media', 'mfn-opts'),
						'class' => 'mfn-info',
					),

					array(
						'id' => 'mfn-post-video',
						'type' => 'text',
						'title' => __('Video | ID', 'mfn-opts'),
						'sub_desc' => __('YouTube or Vimeo', 'mfn-opts'),
						'desc' => __('It`s placed in every YouTube & Vimeo video, for example:<br /><b>YouTube:</b> http://www.youtube.com/watch?v=<u>WoJhnRczeNg</u><br /><b>Vimeo:</b> http://vimeo.com/<u>62954028</u>', 'mfn-opts'),
						'class' => 'small-text mfn-post-format video'
					),
					
					

					// seo

					array(
						'id' => 'mfn-meta-info-seo',
						'type' => 'info',
						'title' => '',
						'desc' => __('SEO <span>below settings overwrite theme options</span>', 'mfn-opts'),
						'class' => 'mfn-info',
					),

					array(
						'id' => 'mfn-meta-seo-title',
						'type' => 'text',
						'title' => __('SEO | Title', 'mfn-opts'),
					),

					array(
						'id' => 'mfn-meta-seo-description',
						'type' => 'text',
						'title' => __('SEO | Description', 'mfn-opts'),
					),

					array(
						'id' => 'mfn-meta-seo-keywords',
						'type' => 'text',
						'title' => __('SEO | Keywords', 'mfn-opts'),
					),

					array(
						'id' => 'mfn-meta-seo-og-image',
						'type' => 'upload',
						'title' => __('Open Graph | Image', 'mfn-opts'),
						'sub_desc' => __('e.g. Facebook share image', 'mfn-opts'),
					),

				),
			);

		}

		/**
		 * Register new post type and related taxonomy
		 */

		public function register()
		{
			$slug = esc_attr(mfn_opts_get('video-slug', 'video-item'));
			$tax = esc_attr(mfn_opts_get('video-tax', 'video-types'));

			$labels = array(
				'name' => esc_html__('Videos', 'mfn-opts'),
				'singular_name' => esc_html__('Video item', 'mfn-opts'),
				'add_new' => esc_html__('Add New', 'mfn-opts'),
				'add_new_item' => esc_html__('Add New Video item', 'mfn-opts'),
				'edit_item' => esc_html__('Edit Video item', 'mfn-opts'),
				'new_item' => esc_html__('New Video item', 'mfn-opts'),
				'view_item' => esc_html__('View Video item', 'mfn-opts'),
				'search_items' => esc_html__('Search Video items', 'mfn-opts'),
				'not_found' => esc_html__('No video items found', 'mfn-opts'),
				'not_found_in_trash' => esc_html__('No video items found in Trash', 'mfn-opts'),
				'parent_item_colon' => '',
			  );

			$args = array(
				'labels' => $labels,
				'menu_icon' => 'dashicons-video-alt3',
				'public' => true,
				'publicly_queryable' => true,
				'show_ui' => true,
				'query_var' => true,
				'capability_type' => 'post',
				'hierarchical' => false,
				'menu_position' => null,
				'show_in_rest' => true,
				'rewrite' => array(
					'slug' => $slug,
				),
				'supports' => array('author', 'comments', 'custom-fields', 'editor', 'excerpt', 'page-attributes', 'thumbnail', 'title'),
			);

			register_post_type('video', $args);

			register_taxonomy('video-types', 'video', array(
				'label' => esc_html__('Video categories', 'mfn-opts'),
				'hierarchical' => true,
				'query_var' => true,
				'show_in_rest' => true,
				'rewrite' => array(
					'slug' => $tax,
				),
			));
		}

		/**
		 * Add new columns to posts screen
		 */

		public function add_columns($columns)
		{
			$newcolumns = array(
				'cb' => '<input type="checkbox" />',
				'video_thumbnail' => esc_html__('Thumbnail', 'mfn-opts'),
				'title' => esc_html__('Title', 'mfn-opts'),
				'video_types' => esc_html__('Categories', 'mfn-opts'),
				'video_order' => esc_html__('Order', 'mfn-opts'),
  		);
			$columns = array_merge($newcolumns, $columns);

			return $columns;
		}

		/**
		 * Custom column on posts screen
		 */

		public function custom_column($column)
		{
			global $post;

			switch ($column) {
				case 'video_thumbnail':
					if (has_post_thumbnail()) {
						the_post_thumbnail('50x50');
					}
					break;
				case 'video_types':
					echo get_the_term_list($post->ID, 'video-types', '', ', ', '');
					break;
				case 'video_order':
					echo esc_attr($post->menu_order);
					break;
			}
		}

	}
}

new Mfn_Post_Type_Video();
